/* This file was generated by syncqt. */
#ifndef QT_QT3DQUICKINPUT_VERSION_H
#define QT_QT3DQUICKINPUT_VERSION_H

#define QT3DQUICKINPUT_VERSION_STR "5.15.18"

#define QT3DQUICKINPUT_VERSION 0x050F12

#endif // QT_QT3DQUICKINPUT_VERSION_H
